<?php

	// 20 min max runtime
	set_time_limit(20 * 60);
	
	if (isConsoleMode()) {
		// text interface
		require_once('newsscraper_config.php');
		require_once('newsscraper_highlight.php');
		define('NL', "\n");
	} else {
		// web interface
		define('NL', "<br>\n");
	}
	
	// web security
	if ((!preg_match(LOCAL_IPS, $_SERVER["REMOTE_ADDR"]) && (!isset($_GET[SECURITY_URL]) || ($_POST['secure'] != SECURITY_URL))) && !isConsoleMode()) {
		// you get nothing!
		exit();
	}
	
	// check if this is readonly
	// ADD CHECK FOR CONSOLE MODE
	if ($_POST["process"] == "PROCESS DOWNLOADS") {
		if (!isConsoleMode() && ((isSabnzbdIdle() && pidCheck()) || NZB_EXPERT_MODE)) {
			DEFINE('NZB_WRITABLE', true);
			echo colorize('Currently in full write mode.', 'red', true) . NL;
			pidWrite();
		} else {
			DEFINE('NZB_WRITABLE', false);
			if (!isSabnzbdIdle()) {
				echo colorize('SABnzbd is currently running/has queued items, staying in read-only mode.', 'lightgreen', true) . NL;
			} elseif (!pidCheck()) {
				echo colorize('pid found, staying in read-only mode.', 'lightgreen', true) . NL;
			}
		}
	} else {
		DEFINE('NZB_WRITABLE', false);
		echo colorize('Currently in read-only mode.', 'lightgreen', true) . NL;
	}
	
	sanityCheck();
	$removedFiles = cleanJunk();
	movefiles();
	pidClear();
	generateDownloadedList();
	if (!isConsoleMode()) {
		// button for web only
		echo NL . '<form action="index.php?processdownloaded" method="POST" id="torrentsForm">';
		if (isset($_GET[SECURITY_URL])) {
			echo '<input type="hidden" name="secure" value="' . SECURITY_URL . '">';
		}
		echo '<input type="submit" name="process" value="PROCESS DOWNLOADS" style="width:100%;"></form>';
	}
	// done
	
	
	// STEPS
	// step 1/4
	function sanityCheck() {
		// make this check all directories someday
		if (!directoryExists($GLOBALS['nzbDownloadDir'])) {
			echo 'nzb download did not found, exiting.' . NL;
			exit();
		}
	}
		
	// step 2/4
	function cleanJunk() {
		echo NL . 'Cleaning junk files and empty directories.' . NL;
		$removedFiles = array();
		$folders = getDirectoryContents($GLOBALS['nzbDownloadDir'], true);
		if ($folders !== false) {
			$folders = array_diff($folders, $GLOBALS['nzbIgnoreFolders']);	// remove folders we dont make changed
			foreach($folders as $folder) {
				// check for which type
				$nzbType = 'ALL';
				foreach($GLOBALS['nzbFolders'] as $type => $dir) {
					if ($folder . '/' == $dir) {
						$nzbType = $type;
						break;
					}
				}
				$subfolders = getDirectoryContents($GLOBALS['nzbDownloadDir'] . $folder, true);
				foreach($subfolders as $subfolder) {
					// check for empty directories
					if (isDirectoryEmpty($GLOBALS['nzbDownloadDir'] . $folder . '/' . $subfolder)) {
						removeDirectory($GLOBALS['nzbDownloadDir'] . $folder . '/' . $subfolder);
						array_push($removedFiles, $GLOBALS['nzbDownloadDir'] . $folder . '/' . $subfolder);
					} else {
						// otherwise check for junk files within the directory
						$files = getDirectoryContents($GLOBALS['nzbDownloadDir'] . $folder . '/' . $subfolder, false, true, true);
						if ($files === false) {
							// either has a directory or no permission
							continue;
						}
						foreach($files as $file) {
							// go through files, remove trash
							foreach($GLOBALS['nzbTargetFilesToDelete'][$nzbType] as $regex) {
								if (preg_match($regex, $file)) {
									deleteFile($GLOBALS['nzbDownloadDir'] . $folder . '/' . $subfolder . '/' . $file);
									array_push($removedFiles, $GLOBALS['nzbDownloadDir'] . $folder . '/' . $subfolder . '/' . $file);
								}
							}
						}
					}
				}
			}
		}
		
		return $removedFiles;	// for use with array_key_exists()
	}

	// step 3/4
	function movefiles() {
		echo NL . 'Moving files.' . NL;
		flush();
		foreach ($GLOBALS['nzbFolders'] as $nzbType => $nzbFolder) {	// get all folders to check
			if (!isset($GLOBALS['nzbTargetDirectories'][$nzbType])) {
				echo '-- no target directory for this type specified, skipping' . NL;
				continue;
			}
			if (!directoryExists($GLOBALS['nzbDownloadDir'] . $nzbFolder, true, false, true)) {
				continue;
			}
			$folders = getDirectoryContents($GLOBALS['nzbDownloadDir'] . $nzbFolder, true, false, true, false);
			if ($folders === false) {
				echo '-- inadequite permission, stopping';
				exit();
			}
			foreach ($folders as $folder) {	// go through each folder
				$sourceFolder = $GLOBALS['nzbDownloadDir'] . $nzbFolder . $folder;
				if (array_search($sourceFolder, $GLOBALS['removedFiles']) !== false) {
					// directory removed
					continue;
				}
				echo colorize($folder, 'yellow', true) . NL;
				echo '-- checking directory';
				$files = getDirectoryContents($sourceFolder, false, true, true, true);
				if ($files === false) {
					echo ', contains a directory, file larger then 2G or inadequite permission, skipping' . NL . NL;
					continue;
				}
				$moved = false;
				echo ', passed' . NL;
				// check if any contents inside dir need to be moved
				if (isset($GLOBALS['nzbTargetExtension'][$nzbType])) {
					foreach($files as $file) {
						foreach($GLOBALS['nzbTargetExtension'][$nzbType] as $extension => $dir) {
							if (preg_match($extension, $file)) {
								$targetFolder = $dir;
								echo '-- file extnesion "' . $file . '" moved to "' . colorize($dir, 'yellow', true) . '"';
							//	if (directoryExists($targetFolder)) {
									echo NL;
									moveFile($sourceFolder . '/' . $file, $targetFolder . $file);
							//	} else {
							//		echo ', directory does not exist, skipping.' . NL;
							//	}
								array_push($GLOBALS['removedFiles'], $sourceFolder . '/' . $file);
							}
						}
					}
				}
				// if files exists in directory - see if directory name matches any of the regex
				foreach ($GLOBALS['matchArray'] as $match) {
					if (isset($GLOBALS['nzbFolders'][$match['type']]) && ($match['type'] == $nzbType) && $match['fullname'] && ($match['match'] !== null)) {	// must be a type we want to move and have a 'fullname'
						if (preg_match("/^" . $match['match'] . "/i", $folder)) {
							if ($match['dir'] === null) {
								$targetFolder = $GLOBALS['nzbTargetDirectories'][$nzbType] . nzbTargetDirectoryName($match['fullname'], $nzbType);
							} else {
								$targetFolder = $match['dir'] . nzbTargetDirectoryName($match['fullname'], $nzbType);
							}
							echo '-- match with "' . $match['fullname'] . '", creating directory "' . dirname($targetFolder) . '/' . colorize(basename($targetFolder), 'yellow', true) . '"';
							if (directoryExists($targetFolder)) {
								echo ', directory exists' . NL;
							} else {
								// CREATE FOLDER
								echo NL;
								createDirectory($targetFolder);
							}
							if (directoryExists($targetFolder, true) || !NZB_WRITABLE) {
								echo '-- moving contents' . NL;
								moveDirectoryContents($sourceFolder, $targetFolder, $nzbType);
								removeDirectory($sourceFolder);
							} else {
								echo '-- inainadequite permission to move into folder, skipping' . NL;
								continue;
							}
							$moved = true;	// whether we did or not, if it matched we dont want it to have a new directory
							break; // ?
						}
					}
				}
				if (!$moved) {
					if (NZB_UNKNOWN_TV_TO_MISC && ($nzbType == TVSHOWS)) {
						// unmatched TV show, move to "/Misc"
						$targetFolder = $GLOBALS['nzbTargetDirectories'][$nzbType] . 'Misc';
						echo '-- unmatched tv show, moving into directory "' . dirname($targetFolder) . '/' . colorize(basename($targetFolder), 'yellow', true) . '"';
					} elseif (!NZB_CREATE_MOVIE_DIRECTORY && ($nzbType == MOVIES)) {
						// movies are just moved to root of movie folder
						$targetFolder = preg_replace("/^(.*)\/$/", "$1", $GLOBALS['nzbTargetDirectories'][$nzbType]);
						echo '-- move to root movie folder "' . dirname($targetFolder) . '/' . colorize(basename($targetFolder), 'yellow', true) . '"';
					} else {
						// nothing above matched, just move the directory itself rather the contents
						$targetFolder = $GLOBALS['nzbTargetDirectories'][$nzbType] . nzbTargetDirectoryName($folder, $nzbType);
						echo '-- moving into directory "' . dirname($targetFolder) . '/' . colorize(basename($targetFolder), 'yellow', true) . '"';
					}
					if (directoryExists($targetFolder)) {
						echo ', directory exists' . NL;
					} else {
						// CREATE FOLDER
						echo NL;
						createDirectory($targetFolder);
					}
					moveDirectoryContents($sourceFolder, $targetFolder, $nzbType);
					removeDirectory($sourceFolder);
				}
				echo NL;
			}
		}
	}
	
	// Step 4/4
	// generate list of downloaded shows
	//<<< make some kind of system for this, this is a complete hacky hardcoded mess
	function generateDownloadedList() {
		if (DISPLAY_DOWNLOADED_LINK && NZB_WRITABLE) {
			echo colorize('Generating updated list of shows', 'lightgreen') . NL;
			if (!file_exists(DOWNLOADED_FILE)) {
				@touch(DOWNLOADED_FILE);
			}
			if (file_exists(DOWNLOADED_FILE) && is_writeable(DOWNLOADED_FILE)) {
				exec(
				'find "/video/TV/" -name "*.avi" >> '.DOWNLOADED_FILE.' ; ' .
				'find "/video/TV/" -name "*.mkv" >> '.DOWNLOADED_FILE.' ; ' .
				'find "/video/TV/" -name "*.mpg" >> '.DOWNLOADED_FILE.' ; ' .
				'find "/video/TV/" -name "*.vob" >> '.DOWNLOADED_FILE.' ; ' .
				'find "/video/TV/" -name "*.wmv" >> '.DOWNLOADED_FILE.' ; ' .
				'find "/video/Movies/" -name "*.avi" >> '.DOWNLOADED_FILE.' ; ' .
				'find "/video/Movies/" -name "*.mkv" >> '.DOWNLOADED_FILE.' ; ' .
				'find "/video/Movies/" -name "*.mpg" >> '.DOWNLOADED_FILE.' ; ' .
				'find "/video/Movies/" -name "*.vob" >> '.DOWNLOADED_FILE.' ; ' .
				'find "//video/Movies/" -name "*.wmv" >> '.DOWNLOADED_FILE.' ; ' .
				'cat '.DOWNLOADED_FILE.' | sort | uniq > /tmp/dlshows.txt && mv -f /tmp/dlshows.txt '.DOWNLOADED_FILE);
			} else {
				echo colorize('File not found or failed permission', 'red') . NL;				
			}
		}
	}
	
	
	// functions
	
	// check if running from the commandline
	function isConsoleMode() {
		return isset($GLOBALS['argv']);
	}

	// check SAB
	function isSabnzbdIdle() {
		// NEVER BOTHERED
		return true;
	}
	
	// file functions
	// get directory contents
	function getDirectoryContents($directory, $directoriesOnly = false, $failIfDirectory = false, $failIfUnwritable = false, $failIfLargeFiles = false) {
		// remove . and ..
		$items = scandir($directory);
		$remove = array('.', '..');
		$items = array_diff($items, $remove);
		if ($directoriesOnly || $failIfDirectory || $failIfLargeFiles) {
			$newItems = array();
			foreach ($items as $item) {
				if (directoryExists($directory . '/' . $item)) {
					if ($failIfDirectory) {
						return false;
					}
					if ($failIfLargeFiles && (getFilesize($directory . '/' . $item, true) === false)) {
						echo '-- ' . colorize('ignoring file larger then 2G', 'orange') . ' "' . $source . '"' . NL;
						return false;
					}
					array_push($newItems, $item);
				}
			}
			if ($directoriesOnly) {
				$items = $newItems;
			}
		}
		if ($failIfUnwritable) {
			foreach ($items as $item) {
				if (!is_writable($directory . '/' . $item)) {
					return false;
				}
			}
		}
		return $items;
	}
	function directoryExists($directory, $failIfUnwriable = false) {
		if (!is_dir($directory)) {
			return false;
		} elseif ($failIfUnwriable && !is_writable($directory)) {
			return false;
		} else {
			return true;
		}
	}
	function createDirectory($directory) {
		echo '-- ' . colorize('create directory', 'lightgreen') . ' "' . $directory . '"' . NL;
		if (!directoryExists($directory) && NZB_WRITABLE) {
			return @mkdir($directory, 0775);
		}
	}
	function removeDirectory($directory) {
		echo '-- ' . colorize('remove directory', 'orange') . ' "' . $directory . '"' . NL;
		if (directoryExists($directory) && NZB_WRITABLE) {
			return @rmdir($directory);
		}
	}
	function deleteFile($file) {
		echo '-- ' . colorize('remove file', 'orange') . ' "' . $file . '" (' . getFilesize($file) . ')' . NL;
		if (NZB_WRITABLE) {
			return @unlink($file);
		}
	}
	function isDirectoryEmpty($directory) {
		if (is_dir($directory)) {
			$files = getDirectoryContents($directory);
			if (($files !== false && (count($files) > 0))) {
				return false;
			}
		}
		return true;
	}
	// move all contents of directories between each other
	// does not check permissions/move folders (checks are done earlier)
	function moveDirectoryContents($sourceDirectory, $targetDirectory, $nzbType) {
		$sourceFiles = getDirectoryContents($sourceDirectory);
		foreach($sourceFiles as $file) {			
			$targetFile = trimFilename($file, $nzbType);
			moveFile($sourceDirectory . '/' . $file, $targetDirectory . '/' . $targetFile);
		}
		//removeDirectory($sourceDirectory);
	}
	// move individual file
	function moveFile($source, $target) {
		if (array_search($source, $GLOBALS['removedFiles']) === false) {
			if (getFilesize($source, true) === false) {
				echo '-- large file ignored "' . $source . '"' . NL;
			} else {
				echo '-- ' . colorize('moving file', 'lightgreen') . ' "' . $target . '" (' . getFilesize($source) . ')' . NL;
				flush();
				if (NZB_WRITABLE) {
					// if file was not already deleted
					@rename($source, $target);
				}
			}
		}
	}
	function trimFilename($filename, $nzbType) {
		if (isset($GLOBALS['nzbTrimFilenames'][$nzbType])) {
			foreach($GLOBALS['nzbTrimFilenames'][$nzbType] as $regex => $replace) {
				$filename = preg_replace($regex, $replace, $filename);
			}
		}
		return $filename;
	}
	function getFilesize($filename, $return_error = false) {
		$size = @filesize($filename);	// need the @ for not visibally puking an error on large files (2G+)
		if ($size === false) {
			if ($return_error) {
				return false;
			}
			return '-';
		} elseif ($size == 0) {
			return '0';
		}
		$sizeArray = array('b', 'k', 'mb', 'g');
		for($i=0; $i<count($sizeArray); $i++) {
			if ($size <= 1024) {
				return round($size, 1) . $sizeArray[$i];
			}
			$size = $size / 1024; 
		}
		return '-';	// should not happen
	}
	
	// PIDs
	function pidCheck() {
		return !file_exists(NZB_PID_NAME);
	}
	function pidWrite() {
		if (NZB_WRITABLE) {
			return @touch(NZB_PID_NAME);	// for now ignore what the actual pid is
		}
		return true;
	}
	function pidClear() {
		if (NZB_WRITABLE) {
			return @unlink(NZB_PID_NAME);
		}
		return true;
	}
	
	// colorization
	function colorize($text, $color = false, $bold = false) {
		if (isConsoleMode()) {
			// need to add console mode as well
			return $text;
		} else {
			$output = '<span style="';
			if ($color !== false) {
				$output .= 'color:' . $color . ';';
			}
			if ($bold !== false) {
				$output .= 'font-weight:bolder;';
			}
			$output .= '">' . $text . '</span>';
			return $output;
		}
	}
	
	
?>
